@@ -12,7 +12,7 @@ import re
 class requirement_module:
     def __init__(self, module_path="", parent_prefix="", root_module=True):
         self.parent_prefix = parent_prefix
-        self.module_path = module_path
+        self.module_path = os.path.abspath(module_path)
         try:
             self.git_repo = git.Repo(self.module_path, search_parent_directories=True)
         except:
@@ -27,7 +27,7 @@ class requirement_module:
 
 
         if self.config['req_version'] < 0.2:
-            if os.path.exists(module_path + '/config.yaml'):
+            if os.path.exists(module_path + '/module-prefix.yaml'):
                 with open(module_path + '/module-prefix.yaml', 'r') as proj_pref_file:
                     self.config['module_prefix'] = yaml.safe_load(proj_pref_file)
             else:
@@ -80,7 +80,7 @@ class requirement_module:
                 for test_file in test_result_files:
                     self.import_test_results(test_file)
         if root_module:
-            for req in self.reqs:
+            for req in self.reqs.copy():
                 self.update_link_status(req)
 
     def read_reqs(self):
@@ -103,35 +103,6 @@ class requirement_module:
             self.reqs.add_node(req_name)
             for field in req.keys():
 
-                # Create links between linked nodes
-                if "links" in field:
-                    links = req[field].split(',')
-                    for link in links:
-                        if ':' in link:
-                            link_type, linked_req = link.split(':')
-                            link_type = link_type.strip()
-                            # Don't add project prefix to extern links
-                            if 'extern' not in link_type:
-                                linked_req = self.parent_prefix + '_' + linked_req.strip()
-
-                            # Verify that the requirement is not linked two ways.
-                            uptest = (linked_req, req_name) in self.reqs.edges.keys(
-                            ) and self.reqs.edges[(linked_req, req_name)]['type'] == link_type
-                            downtest = (req_name, linked_req) in self.reqs.edges.keys(
-                            ) and self.reqs.edges[(req_name, linked_req)]['type'] == link_type
-                            assert(not (uptest and downtest))
-
-                            if "upward" in field:
-                                self.reqs.add_edge(
-                                    linked_req, req_name, type=link_type)
-                            else:
-                                self.reqs.add_edge(
-                                    req_name, linked_req, type=link_type)
-                            self.reqs.nodes[linked_req]['non_stored_fields'] = {}
-                            self.reqs.nodes[linked_req]['non_stored_fields']['Internal'] = False
-                            self.reqs.nodes[linked_req]['non_stored_fields']['color'] = 'gray'
-                            self.reqs.nodes[linked_req]['non_stored_fields']['link_status_updated'] = False
-
                 if not field in fields:
                     fields.append(field)
 
@@ -149,6 +120,43 @@ class requirement_module:
 
     def update_link_status(self, req, subgraph=None):
         if not self.reqs.nodes[req]['non_stored_fields']['link_status_updated']:
+            for field in ['upward_links', 'downward_links']:
+                # Create links between linked nodes
+                if field in self.reqs.nodes[req].keys():
+                    links = self.reqs.nodes[req][field].split(',')
+                    for link in links:
+                        if ':' in link:
+                            link_type, linked_req = link.split(':')
+                            link_type = link_type.strip()
+                            linked_req = linked_req.strip()
+                            # Don't add project prefix to extern links
+                            if 'extern' not in link_type:
+                                # Search if req is existing on any level
+                                for d in range(1, len(req.split('_'))+1):
+                                    linked_req_full = '_'.join(req.split('_')[:-d] + [linked_req])
+                                    if linked_req_full in self.reqs.nodes.keys():
+                                        break
+
+                                linked_req = linked_req_full
+
+                            # Verify that the requirement is not linked two ways.
+                            uptest = (linked_req, req) in self.reqs.edges.keys(
+                            ) and self.reqs.edges[(linked_req, req)]['type'] == link_type
+                            downtest = (req, linked_req) in self.reqs.edges.keys(
+                            ) and self.reqs.edges[(req, linked_req)]['type'] == link_type
+                            assert (not (uptest and downtest))
+
+                            if "upward" in field:
+                                self.reqs.add_edge(
+                                    linked_req, req, type=link_type)
+                            else:
+                                self.reqs.add_edge(
+                                    req, linked_req, type=link_type)
+                            self.reqs.nodes[linked_req]['non_stored_fields'] = {}
+                            self.reqs.nodes[linked_req]['non_stored_fields']['Internal'] = False
+                            self.reqs.nodes[linked_req]['non_stored_fields']['color'] = 'gray'
+                            self.reqs.nodes[linked_req]['non_stored_fields']['link_status_updated'] = False
+
             if subgraph is None:
                 subgraph = self.reqs
 
@@ -196,8 +204,7 @@ class requirement_module:
 
             self.reqs.nodes[req]['non_stored_fields']['link_status_updated'] = True
 
-	
-	# Propagates the reference of the full project graph to all modules
+    # Propagates the reference of the full project graph to all modules
     # This is so that changes can be done on any level, and all requirements shall only exist once.
     def update_to_root_graph(self, root_reqs):
         self.reqs = root_reqs
@@ -231,7 +238,7 @@ class requirement_module:
         return '_'.join(name.split('_')[1:])
 
     def add_req(self, req, position=-1):
-        if req['Req-Id'] == "":
+        if not req['Req-Id']:
             if self.config['req_number_format'] == 'time_hash':
                 hash = hashlib.sha1()
                 while True:
@@ -346,7 +353,13 @@ class requirement_module:
 
                 match = re.search(pattern, case.name)
                 if match:
-                    testname = '_'.join([self.module_prefix.split('_')[0],match.groups()[0]])
+                    # Search if test is existing on any level
+                    testname_full = '_'.join([self.module_prefix, match.groups()[0]])
+                    for d in range(1, len(self.module_prefix.split('_')) + 1):
+                        if testname_full in self.reqs.nodes.keys():
+                            break
+                        testname_full = '_'.join(self.module_prefix.split('_')[:-d] + [match.groups()[0]])
+                    testname = testname_full
                     linktype = match.groups()[1]
                     # Ok for nx to add node that already exists
                     self.reqs.add_node(testname + '_result', result=result,
@@ -354,12 +367,12 @@ class requirement_module:
                     self.reqs.nodes[testname + '_result']['Req-Id'] = testname.split('_')[-1] + '_result'
                     self.reqs.add_edge(testname, testname + '_result', type='Test-Result')
                 elif connect_with_naming_convention:
-                    self.reqs.add_node(case.name + '_result', result=result,
+                    self.reqs.add_node(case.name, result=result,
                                    non_stored_fields={'color': color, 'link_status_updated': True}, Type='Test-Result')
 
                     for req_name, req_content in self.reqs.nodes.items():
                         if 'Description' in req_content.keys() and case.name in req_content['Description']:
-                            self.reqs.add_edge(req_name, case.name)
+                            self.reqs.add_edge(req_name, case.name, type='Test-Result')
 
     def get_related_reqs(self, req_name, subgraph=None):
         if subgraph:
@@ -378,8 +391,10 @@ class requirement_module:
             nodes.extend([n for n, d in self.reqs.nodes.items() if field[0] in d.keys() and d[field[0]] == field[1]])
         return self.reqs.subgraph(nodes)
 
-def init_module(parent_path, module_name, module_prefix, req_numbering='time_hash'):
+def init_module(parent_path, module_name, module_prefix, req_numbering='time_hash', root_module=False):
     module_path = parent_path + '/' + module_name
+    git_repo = git.Repo(parent_path, search_parent_directories=True)
+
     if not os.path.exists(module_path):
         os.mkdir(module_path)
     assert(not module_name == '')
@@ -395,18 +410,24 @@ def init_module(parent_path, module_name, module_prefix, req_numbering='time_has
     config['modules'] = []
     config['source_paths'] = []
     config['source_extensions'] = []
+    config['root_module'] = root_module
     next_id = 1
 
     with open(module_path + '/config.yaml', 'w') as config_file:
         yaml.dump(config, config_file)
+        git_repo.git.add(config_file.name)
     with open(module_path + '/next-id.yaml', 'w') as next_id_file:
         yaml.dump(next_id, next_id_file)
+        git_repo.git.add(next_id_file.name)
     with open(module_path + '/used-ids.yaml', 'w') as used_ids_file:
         yaml.dump([], used_ids_file)
+        git_repo.git.add(used_ids_file.name)
     with open(module_path + '/modules.yaml', 'w') as modules_file:
         yaml.dump([], modules_file)
+        git_repo.git.add(modules_file.name)
     with open(module_path + '/reqs.yaml', 'w') as reqs_file:
         yaml.dump([], reqs_file)
+        git_repo.git.add(reqs_file.name)
 
     if os.path.exists(module_path + '/../config.yaml'):
         with open(module_path + '/../config.yaml', 'r') as parent_config_file:
